{% extends "fastapi_aad_auth.ui:base.html" %}
{% block Content %}
        <main class='container' role="main">
        {% block ContentHeader %}
            {% if logo %}
                {{logo | safe }}
            {% endif %}
            {% if appname %}
                <h1 class="h1">{{appname}}</h1>
            {% endif %}
        {% endblock ContentHeader %}
        {% block Error %}
            <h2>Server Error {{status_code}}:</h2>
            <h3> {{error_type}}</h3>
            <p class="lead">{{error_description}}</p>
            <div class="card card-grey ">
                <div class="card-header">
                    <h7 class="card-title">
                        Detailed Error Message:
                    </h7>
                </div>
                <code class="language-json card-body">
                        {{ error | safe }}
                {% if error_detail %}
                        <br>
                        {{ error_detail }}
                {% endif %}
            </code>
            </div>
        {% endblock Error %}
        {% block ContentFooter %}
            {% if copyright %}
                <p class="mt-5 mb-3 lead text-muted">© {{ copyright }}</p>
            {% endif %}
            {% if explanation %}
                <p class="mt-5 mb-3 lead text-muted">{{ explanation }}</p>
            {% endif %}
        {% endblock ContentFooter %}
        </main>
{% endblock Content %}
