<!DOCTYPE html>
<html lang="en">
    <head>
    {% block Head %}
        <title>{{app_name}}</title>
        <meta charset="UTF-8">
        <meta http-equiv="X-UA-Compatible" content="IE=edge">
        <meta name="viewport" content="width=device-width, initial-scale=1">
        {% block CSS %}
        <link href="{{static_path}}/css/bootstrap.min.css" rel="stylesheet">
        <link rel="stylesheet" href="{{static_path}}/css/cover.css">
        {% endblock CSS %}
        {% block Favicon %}
        <link rel="icon" type="img/ipng" href="/static/img/favicon.png" />
        {% endblock Favicon %}
        <meta http-equiv="Content-Type" content="text/html;charset=iso-8859-1" >
        {% block HeadScripts %}
        <script type="text/javascript" src="https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js"></script>
        {% endblock HeadScripts %}
    {% endblock Head %}
    </head>
    <body>
    {% block Body %}
        {% block Nav %}
        {% endblock Nav %}
        {% block Content %}
        {% endblock Content %}
        {% block Footer %}
        {% endblock Footer %}
        {% block BodyScripts %}
        {% endblock BodyScripts %}
    {% endblock Body %}
    </body>
</html>